﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.ScheduleView;

namespace LightSwitchApplication
{
	public class ViewModel : ViewModelBase
	{
		private DelegateCommand setCategoryCommand;
		private DelegateCommand setTimeMarkerCommand;

		private DelegateCommand setTodayCommand;
		private DelegateCommand setWorkWeekCommand;

		private Appointment selectedAppointment;
		private bool showNewsChannel = true;
		private bool showEntertainmentChannel = true;
		private bool showSportsChannel = true;
		private GroupDescriptionCollection groupDescriptions;
		private Func<object, bool> groupFilter;
		private DateTime currentDate;
		private ViewDefinitionBase activeViewDefinition;

		private List<Appointment> appointments;

		public ViewModel()
		{
			this.SetCategoryCommand = new DelegateCommand(this.SetCategoryCommandExecuted, this.SetCategoryCommandCanExecute);
			this.SetTimeMarkerCommand = new DelegateCommand(this.SetTimeMarkerCommandExecuted, this.SetTimeMarkerCommandCanExecute);
			this.SetTodayCommand = new DelegateCommand(this.SetTodayCommandExecuted, this.SetTodayCommandCanExecute);
			this.SetWorkWeekCommand = new DelegateCommand(this.SetWorkWeekCommandExecuted, this.SetWorkWeekCommandCanExecute);
			this.CurrentDate = DateTime.Today;
		}

		public List<Appointment> Appointments
		{
			get
			{
				return this.appointments;
			}
			set
			{
				this.appointments = value;
				this.OnPropertyChanged(() => this.Appointments);
			}
		}

		public DelegateCommand SetCategoryCommand
		{
			get
			{
				return this.setCategoryCommand;
			}
			set
			{
				this.setCategoryCommand = value;
			}
		}

		public DelegateCommand SetTimeMarkerCommand
		{
			get
			{
				return this.setTimeMarkerCommand;
			}
			set
			{
				this.setTimeMarkerCommand = value;
			}
		}

		public DelegateCommand SetTodayCommand
		{
			get
			{
				return this.setTodayCommand;
			}
			set
			{
				this.setTodayCommand = value;
			}
		}

		public DelegateCommand SetWorkWeekCommand
		{
			get
			{
				return this.setWorkWeekCommand;
			}
			set
			{
				this.setWorkWeekCommand = value;
			}
		}

		public Appointment SelectedAppointment
		{
			get
			{
				return this.selectedAppointment;
			}
			set
			{
				this.selectedAppointment = value;
			}
		}

		public bool ShowNewsChannel
		{
			get
			{
				return this.showNewsChannel;
			}
			set
			{
				if (this.showNewsChannel != value)
				{
					this.showNewsChannel = value;
					this.OnPropertyChanged(() => this.ShowNewsChannel);
					this.UpdateGroupFilter();
				}
			}
		}

		public bool ShowEntertainmentChannel
		{
			get
			{
				return this.showEntertainmentChannel;
			}
			set
			{
				if (this.showEntertainmentChannel != value)
				{
					this.showEntertainmentChannel = value;
					this.OnPropertyChanged(() => this.ShowEntertainmentChannel);
					this.UpdateGroupFilter();
				}
			}
		}

		public bool ShowSportsChannel
		{
			get
			{
				return this.showSportsChannel;
			}
			set
			{
				if (this.showSportsChannel != value)
				{
					this.showSportsChannel = value;
					this.OnPropertyChanged(() => this.ShowSportsChannel);
					this.UpdateGroupFilter();
				}
			}
		}

		public GroupDescriptionCollection GroupDescriptions
		{
			get
			{
				if (this.groupDescriptions == null)
				{
					this.groupDescriptions = new GroupDescriptionCollection() { new DateGroupDescription() };
					this.UpdateGroupDescriptions();
				}
				return this.groupDescriptions;
			}
		}

		public Func<object, bool> GroupFilter
		{
			get
			{
				return this.groupFilter;
			}
			set
			{
				this.groupFilter = value;
				this.OnPropertyChanged(() => this.GroupFilter);
			}
		}

		public DateTime CurrentDate
		{
			get
			{
				return this.currentDate;
			}
			set
			{
				this.currentDate = value;
				this.OnPropertyChanged(() => this.CurrentDate);
			}
		}

		public ViewDefinitionBase ActiveViewDefinition
		{
			get
			{
				return this.activeViewDefinition;
			}
			set
			{
				this.activeViewDefinition = value;
				this.OnPropertyChanged(() => this.ActiveViewDefinition);
			}
		}

		public void SetCategoryCommandExecuted(object parameter)
		{
			if (this.SelectedAppointment == null)
			{
				RadWindow.Alert("Please, select an appointment first.");
			}
			else
			{
				Appointment appointmentToEdit = (from app in this.Appointments where app.Equals(this.SelectedAppointment) select app).FirstOrDefault();
				Category c = parameter as Category;
				appointmentToEdit.Category = c;
				this.Appointments.Remove(appointmentToEdit);
				this.Appointments.Add(appointmentToEdit);
			}
		}

		public void SetTimeMarkerCommandExecuted(object parameter)
		{
			if (this.SelectedAppointment == null)
			{
				RadWindow.Alert("Please, select an appointment first.");
			}
			else
			{
				Appointment appointmentToEdit = (from app in this.Appointments where app.Equals(this.SelectedAppointment) select app).FirstOrDefault();
				TimeMarker t = parameter as TimeMarker;
				appointmentToEdit.TimeMarker = t;
				this.Appointments.Remove(appointmentToEdit);
				this.Appointments.Add(appointmentToEdit);
			}
		}

		public void SetTodayCommandExecuted(object parameter)
		{
			this.CurrentDate = DateTime.Today;
		}

		public void SetWorkWeekCommandExecuted(object parameter)
		{
			WorkWeekViewDefinition workWeek = new WorkWeekViewDefinition();
			this.ActiveViewDefinition = workWeek;
		}

		public bool CheckAppointmentSelection()
		{
#if WPF
			return this.SelectedAppointment != null;
#else
			return true;
#endif
		}

		public bool SetCategoryCommandCanExecute(object parameter)
		{
			return this.CheckAppointmentSelection();
		}

		public bool SetTodayCommandCanExecute(object parameter)
		{
			return true;
		}

		public bool SetWorkWeekCommandCanExecute(object parameter)
		{
			return true;
		}

		public bool SetTimeMarkerCommandCanExecute(object parameter)
		{
			return this.CheckAppointmentSelection();
		}

		private void UpdateGroupFilter()
		{
			this.GroupFilter = new Func<object, bool>(this.GroupFilterFunc);
		}

		private bool GroupFilterFunc(object groupName)
		{
			IResource resource = groupName as IResource;

			return resource == null ? true : this.GetEnabledGroups().Contains(resource.ResourceName, StringComparer.OrdinalIgnoreCase);
		}

		private IEnumerable<string> GetEnabledGroups()
		{
			List<string> enabledGroups = new List<string>();

			if (this.ShowNewsChannel) enabledGroups.Add("LiveCastNews");
			if (this.ShowEntertainmentChannel) enabledGroups.Add("Voozy");
			if (this.ShowSportsChannel) enabledGroups.Add("Sportix");

			return enabledGroups;
		}

		private void UpdateGroupDescriptions()
		{
			ResourceGroupDescription groupDescription = new ResourceGroupDescription();
			groupDescription.ResourceType = "TV";
			this.GroupDescriptions.Add(groupDescription);
		}
	}
}
